package com.axonivy;

import java.io.IOException;

import org.apache.commons.lang3.StringUtils;

import ch.ivyteam.ivy.environment.Ivy;

import com.axonivy.sample.Person;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.deser.std.StdScalarDeserializer;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.std.StdScalarSerializer;

public class CustomBusinessDataModule extends SimpleModule {

	private static final long serialVersionUID = 1L;
	private static final String SEPARATOR = "|";

	public CustomBusinessDataModule()
	{
		Ivy.log().info("registered custom module!");
		addSerializer(Person.class, new CustomPersonSerializer());
		addDeserializer(Person.class, new CustomPersonDeSerializer());
	}
	
	@SuppressWarnings("serial")
	private static class CustomPersonSerializer extends StdScalarSerializer<Person>
	{

		public CustomPersonSerializer() {
			super(Person.class);
		}

		@Override
		public void serialize(Person value, JsonGenerator gen,
				SerializerProvider provider) throws IOException {
			gen.writeString(value.getFirstname()+SEPARATOR+value.getLastname());
		}
	}
	
	@SuppressWarnings("serial")
	private static class CustomPersonDeSerializer extends StdScalarDeserializer<Person>
	{
		public CustomPersonDeSerializer()
		{
			super(Person.class);
		}

		@Override
		public Person deserialize(JsonParser p, DeserializationContext ctxt)
				throws IOException, JsonProcessingException {
			String value = p.readValueAs(String.class);
			if (!StringUtils.contains(value, SEPARATOR))
			{
				// old version.... -> take special care!
				return new Person();
			}
			
			String[] parts = StringUtils.split(value, SEPARATOR);
			Person person = new Person();
			person.setFirstname(parts[0]);
			person.setLastname(parts[1]);
			return person;
		}
	}
	
}
